@@ -12,16 +12,58 @@ from django.template.response import TemplateResponse
 from django.urls import path
 from django.utils.html import format_html
 
-from .models import Rule, RulePath
-from .utils import RULE_ACTION, LockoutException, get_default_action, get_ip
+from .models import Rule, RuleNetwork, RulePath, RuleRatelimit, RuleSource
+from .utils import RULE_ACTION, LockoutException, get_ip, parse_ipaddress
+
+try:
+    import django_fast_ratelimit as ratelimit
+except ImportError:
+    ratelimit = None
+
 
 # Register your models here.
 
 
-class RulePathInlineAdmin(admin.TabularInline):
-    model = RulePath
+class RuleSubMixin:
+    def get_urls(self):
+        return [
+            path("<str:object_id>/change/", self.redirect_change),
+            *super().get_urls(),
+        ]
+
+    def redirect_change(self, request, object_id):
+        rulep = RulePath.objects.get(id=object_id)
+        return HttpResponseRedirect(f"../../../rule/{rulep.rule_id}/change/")
+
+    def has_add_permission(self, request):
+        return False
+
+    def has_change_permission(self, request, obj=None):
+        return False
+
+
+class ExtraOnlyOnInitialMixin:
     extra = 1
 
+    def get_extra(self, request, obj=None, **kwargs):
+        return self.extra if not obj else 0
+
+
+class RulePathInlineAdmin(ExtraOnlyOnInitialMixin, admin.StackedInline):
+    model = RulePath
+
+
+class RuleRatelimitInlineAdmin(ExtraOnlyOnInitialMixin, admin.StackedInline):
+    model = RuleRatelimit
+
+
+class RuleNetworkInlineAdmin(ExtraOnlyOnInitialMixin, admin.TabularInline):
+    model = RuleNetwork
+
+
+class RuleSourceInlineAdmin(ExtraOnlyOnInitialMixin, admin.TabularInline):
+    model = RuleSource
+
 
 # reuse the existing form to prevent form nesting
 _position_template = """
@@ -66,12 +108,17 @@ _position_template = """
 
 @admin.register(Rule)
 class RuleAdmin(admin.ModelAdmin):
-    list_display = ("position_short", "position_buttons", "action", "name", "rule")
+    list_display = ("position_short", "position_buttons", "action", "name")
     list_display_links = ("position_short",)
-    list_editable = ("name", "action", "rule")
+    list_editable = ("name", "action")
     ordering = ("position",)
-    fields = ["name", "action", "rule"]
-    inlines = [RulePathInlineAdmin]
+    fields = ["name", "action"]
+    inlines = [
+        RuleNetworkInlineAdmin,
+        RuleSourceInlineAdmin,
+        RulePathInlineAdmin,
+        RuleRatelimitInlineAdmin,
+    ]
 
     @admin.display(description="")
     def position_short(self, obj):
@@ -84,11 +131,6 @@ class RuleAdmin(admin.ModelAdmin):
             object_id=obj.id,
         )
 
-    def get_changeform_initial_data(self, request):
-        initial = super().get_changeform_initial_data(request)
-        initial["rule"] = "*"
-        return initial
-
     def get_urls(self):
         return [
             path("test_rules/", self.test_rules),
@@ -133,14 +175,25 @@ class RuleAdmin(admin.ModelAdmin):
         return HttpResponseRedirect("../../")
 
     def simulate_rules(self, request, test_path):
-        rule_id = RulePath.objects.match_ip_and_path(get_ip(request), test_path)
-        if rule_id:
-            rule = Rule.objects.get(id=rule_id)
-            if rule.action == RULE_ACTION.deny.value:
-                raise PermissionDenied()
+        rule_id, action, is_catch_all, ratelimits = RulePath.objects.match_ip_and_path(
+            get_ip(request), test_path
+        )
 
-        elif get_default_action() == RULE_ACTION.deny.value:
+        if ratelimit:
+            for rdict in ratelimits:
+                r = ratelimit.get_ratelimit(
+                    request=request,
+                    action=ratelimit.Action.PEEK,
+                    group=rdict["group"],
+                    key=rdict["key"],
+                    rate=rdict["rate"],
+                )
+                r.decorate_object(
+                    request, name=rdict["decorate_name"], block=rdict["block"]
+                )
+        if action == RULE_ACTION.deny:
             raise PermissionDenied()
+
         return HttpResponse(f"accessed: {test_path}")
 
     def test_rules(self, request):
@@ -152,17 +205,17 @@ class RuleAdmin(admin.ModelAdmin):
 
         test_path = request.POST.get("test_path", None) or ""
         if test_path:
-            rule_id = RulePath.objects.match_ip_and_path(test_ip, test_path)
+            rule_id = RulePath.objects.match_ip_and_path(test_ip, test_path)[0]
             self.message_user(
                 request,
-                f"Parameters: ip: {test_ip}, path: {test_path}",
+                f"Parameters: ip: {test_ip} (parsed: {parse_ipaddress(test_ip)}), path: {test_path}",
                 level=INFO,
             )
         else:
-            rule_id = Rule.objects.match_ip(test_ip)
+            rule_id = Rule.objects.match_ip(test_ip)[0]
             self.message_user(
                 request,
-                f"Parameters: ip: {test_ip}",
+                f"Parameters: ip: {test_ip} (parsed: {parse_ipaddress(test_ip)})",
                 level=INFO,
             )
         if rule_id:
@@ -230,21 +283,18 @@ class RuleAdmin(admin.ModelAdmin):
 
 
 @admin.register(RulePath)
-class RulePathAdmin(admin.ModelAdmin):
-    list_display = ("rule", "path", "is_regex")
+class RulePathAdmin(RuleSubMixin, admin.ModelAdmin):
+    list_display = ("rule", "path", "is_regex", "is_active")
+    ordering = ("rule", "id")
 
-    def get_urls(self):
-        return [
-            path("<str:object_id>/change/", self.redirect_change),
-            *super().get_urls(),
-        ]
 
-    def redirect_change(self, request, object_id):
-        rulep = RulePath.objects.get(id=object_id)
-        return HttpResponseRedirect(f"../../../rule/{rulep.rule_id}/change/")
+@admin.register(RuleNetwork)
+class RuleNetworkAdmin(RuleSubMixin, admin.ModelAdmin):
+    list_display = ("rule", "network", "is_active")
+    ordering = ("rule", "id")
 
-    def has_add_permission(self, request):
-        return False
 
-    def has_change_permission(self, request, obj=None):
-        return False
+@admin.register(RuleSource)
+class RuleSourceAdmin(RuleSubMixin, admin.ModelAdmin):
+    list_display = ("rule", "generator_fn", "is_active")
+    ordering = ("rule", "id")
